<?php

namespace App\Exports;

use Maatwebsite\Excel\Concerns\FromArray;
use Maatwebsite\Excel\Concerns\WithHeadings;
use Maatwebsite\Excel\Concerns\ShouldAutoSize;
use Maatwebsite\Excel\Concerns\WithEvents;
use Illuminate\Support\Facades\Session;
use Maatwebsite\Excel\Events\AfterSheet;
use App\Models\VehicleTrakingReportModel;

class VehicleTypeWiseWeightExport implements FromArray,WithHeadings,ShouldAutoSize,WithEvents
{
    protected $data;
    protected $headings;
    protected $columnWidth = [];
    protected $rowHeight = [];
    protected $mergeCells = [];
    protected $font = [];
    protected $fontSize = [];
    protected $bold = [];
    protected $background = []; 
    protected $vertical = []; 
    protected $sheetName;
    protected $borders = [];

    /**
    * @return \Illuminate\Support\Collection
    */
    public function array(): array
    {
        $aData = array();
        $aData['excel'] = array();

        $aData['start_date']      = Session::has('start_date') ? Session::get('start_date') : date('d-m-Y',strtotime('-30 days'));
        $aData['end_date']        = Session::has('end_date') ? Session::get('end_date') : date('d-m-Y',strtotime('now'));
        $aData['vehicle_type']    = session::has('vehicle_type') ? session::get('vehicle_type') : '';

		$aData['records'] = VehicleTrakingReportModel::get_veh_type_wise_weight_mngmt($aData);

        //dd($records);

        // foreach($aData['records'] as $val){
        //     $aData['excel'][] = array(
        //         'date' => $val->date,
        //         'net_weight' => $val->net_weight
        //     );     
        // }
        // return $aData['excel'];
        $total_weight_tons  = 0;
        $grand_total_weight = 0;
        $total_tons = 0;
        if( $aData['records']){
            foreach($aData['records'] as $key=>$aValue){
                $total_weight_tons = ($aValue['net_weight']/1000);
                $total_tons += $total_weight_tons;
                $grand_total_weight += $aValue['net_weight'];
                $aData['excel'][] = array(
                    'date' => $aValue['date'],
                    'net_weight' =>$aValue['net_weight'],
                    'tons' => $total_weight_tons,
                ); 
            }
            $aData['excel'][] = array(
                'Total' => "Total",
                'grand_total_weight' => $grand_total_weight,
                'total_tons' => $total_tons
            ); 
        }else{
            $aData['excel'][] = array(
            );     
        }
        return $aData['excel'];

    }

    public function headings(): array
    {   
        $aData['start_date']      = Session::has('start_date') ? Session::get('start_date') : date('d-m-Y',strtotime('-30 days'));
        $aData['end_date']        = Session::has('end_date') ? Session::get('end_date') : date('d-m-Y',strtotime('now'));
        $aData['vehicle_type']    = session::has('vehicle_type') ? session::get('vehicle_type') : '';
        
        if($aData['vehicle_type'] == 10){
            $weight_type =  'Door to Door';
        }elseif($aData['vehicle_type'] == 4){
            $weight_type = 'Garden';
        }elseif($aData['vehicle_type'] == 5){
            $weight_type = 'Debris';  
        }else{
            $weight_type = 'Door to Door / Garden / Debris';
        }

        return [
                ['Report Name: Vehicle Type-wise Weight Management Report.'],
                ['Start Date : '.$aData['start_date'] ],
                ['End Date : '.$aData['end_date'] ],
                ['Weight Type : '.$weight_type ],
                [],
                [ 
                   'Date ',
                   'Total Net Weight',
                   'Tons' 
                ]
        ];
    }


    public function registerEvents(): array
    {
        return [
            AfterSheet::class => function(AfterSheet $event) {
                $event->sheet->getDelegate()->getStyle('A1:Z1265')->getAlignment()->setHorizontal('left');
                $event->sheet->mergeCells('A1:C1');
                $event->sheet->mergeCells('A2:C2');
                $event->sheet->mergeCells('A3:C3');
                //$event->sheet->mergeCells('A4:C4');
               
                $aData = array();
                $aData['start_date']      = Session::has('start_date') ? Session::get('start_date') : date('d-m-Y',strtotime('-30 days'));
                $aData['end_date']        = Session::has('end_date') ? Session::get('end_date') : date('d-m-Y',strtotime('now'));
                $aData['vehicle_type']    = session::has('vehicle_type') ? session::get('vehicle_type') : '';
                
                $aData['records'] = VehicleTrakingReportModel::get_veh_type_wise_weight_mngmt($aData);

                $i = 7;
                if( $aData['records']){
                    $data = count($aData['records']);
                    $i += $data;
                    $event->sheet->getStyle('A'.$i.':Z'.$i)->applyFromArray([
                        'font'=>[
                                'bold'=>true,
                        ]
                    ]);
                    $event->sheet->getDelegate()->getStyle('A'.$i.':A'.$i)->getAlignment()->setHorizontal('right');
                }
               
                $event->sheet->getStyle('A1:I6')->applyFromArray([
                    'font'=>[
                            'bold'=>true,
                    ]
                ]);

                $event->sheet->getDelegate()->getRowDimension('1')->setRowHeight(25);
                $event->sheet->getDelegate()->getRowDimension('2')->setRowHeight(25);
                $event->sheet->getDelegate()->getRowDimension('3')->setRowHeight(25);
                $event->sheet->getDelegate()->getRowDimension('4')->setRowHeight(25);
                

                $event->sheet->getStyle('A1:Z6')->applyFromArray([
                    'font'=>[
                            'bold'=>true,
                    ]
                ]);

                //display images
                $workSheet = $event->sheet->getDelegate();
                // foreach ($this->getDrawings() as $drawing) {
                //     $drawing->setWorksheet($workSheet);
                // }
            },
        ];
    }
  
}

