<?php

namespace App\Http\Controllers;

use App\Exports\DateWiseExport;
use App\Exports\DaysWiseExport;
use App\Models\ReportModel;
use Illuminate\Support\Facades\Session;
use App\Libraries\Mysecurity;
use App\Models\PakageModel;
use Illuminate\Support\Facades\DB;
use App\Support\Collection;
use Illuminate\Http\Request;
use App\Exports\EmployeeExport;
use App\Exports\EmployeeAttendanceExport;
use App\Exports\EmployeeWiseExport;
use App\Exports\HoursWiseExport;
use App\Exports\InvalidEntriesExport;
use App\Exports\PresentAbsentExport;
use Maatwebsite\Excel\Facades\Excel;

class AttendanceReportController extends Controller
{
    public function __construct()
    {
        $module_link = request()->segments(0);
        if(isset($module_link[0])){
            $module_link = $module_link[0];
            $this->mysecurity = new Mysecurity();
            $this->module_id = $this->mysecurity->get_module_id_by_name($module_link);
            $this->middleware(function ($request, $next){
                $this->mysecurity->check_access(Session::get('logged_in')['id'],$this->module_id,1);       
                return $next($request);
            });
        }
    }

    public function clear_search($method=''){
        session::forget('Pakage_id');
        session::forget('start_date');
        session::forget('end_date');
        session::forget('attendance_date');
        
        session::forget('emp_id');
        return redirect('reports/'.$method);
    }
    
    public function index(){
        $aData = array();
        $aData['result'] = ReportModel::all();
        return view('reports.reports', $aData);
    }

    public function all_employees(Request $request){
        $aData = array();
        $aData['search'] = '';
        if($request->form_type=='search_frm'){
            session(['Pakage_id' => $request->Pakage_id]);
         }
         
        $aData['Pakage_id'] = session::has('Pakage_id') ? session::get('Pakage_id') : "";
        if(!empty($aData['Pakage_id'])){ 
            $aData['search'] = 1;
        }

        $aData['aEmployee'] = ReportModel::get_all_employee($aData)->orderBy('id', 'ASC')->paginate(10);
        // dd($aData['aEmployee']);
        $aData['aPakage'] = PakageModel::get_Pakages()->get();
        return view('reports.all_employees', $aData); 
    }

    public function all_employees_export_excel(){
        return Excel::download(new EmployeeExport(), 'AllEmployee.xlsx');
    }

    public function employee_attendance(Request $request){
        $aData = array();
        $aData['search'] = '';
        if($request->form_type=='search_frm'){
            session(['Pakage_id' => $request->Pakage_id]);
            
            session(['attendance_date' => $request->attendance_date]);
        }
        if($request->form_type=='invalidate'){
			ReportModel::reportInvalid($request->problem);
            echo "<script>alert('Employee attendance marked as invalid successfully')</script>";
        }
         
        $aData['Pakage_id'] = session::has('Pakage_id') ? session::get('Pakage_id') : 0;
        //$aData['start_date'] = session::has('start_date') ? session::get('start_date') : date('d-m-Y',strtotime('-1 month'));
        $aData['attendance_date'] = session::has('attendance_date') ? session::get('attendance_date') :date('d-m-Y');

        if(!empty($aData['Pakage_id'])){ 
            $aData['search'] = 1;
        }
    
        $aData['aPakage'] = PakageModel::get_Pakages()->get();
        $aData['aAttendance'] = ReportModel::get_employee_attendance($aData)->paginate(config('custom.page_limit'));
        return view('reports.employee_attendance', $aData);
    }

    public function employees_attendance_export_excel(){
        return Excel::download(new EmployeeAttendanceExport(), 'EmployeeAttendance.xlsx');
    }

    public function present_report(Request $request){
        $aData = array();
        $aData['search'] = '';
        if($request->form_type=='search_frm'){
            session(['Pakage_id' => $request->Pakage_id]);
            session(['start_date' => $request->start_date]);
            session(['end_date' => $request->end_date]);
        }
         
        $aData['Pakage_id'] = session::has('Pakage_id') ? session::get('Pakage_id') : 0;
        $aData['start_date'] = session::has('start_date') ? session::get('start_date') : date('d-m-Y',strtotime('-1 month'));
        $aData['end_date'] = session::has('end_date') ? session::get('end_date') : date('d-m-Y');

        if(!empty($aData['Pakage_id'])){ 
            $aData['search'] = 1;
        }
        	
		$holidays = array();

		$workin_day = ReportModel::getWorkingDays($aData['start_date'],$aData['end_date'],$holidays);
        // dd($workin_day );
        $aEmployeeData = ReportModel::getPresentData($aData);

        foreach($aEmployeeData as $key=>$val){
            $sql = 'select Pakage_name from admin_pakages where id ='.$val->Pakage_id;
            $rRes = DB::select( DB::raw($sql));	
            $aEmployeeData[$key]->Pakage_name = $rRes[0]->Pakage_name;
			$aEmployeeData[$key]->absent_day = $workin_day - $val->present_day;
			$aEmployeeData[$key]->working_day = $workin_day;
		}

		$aData['aPresent'] = $aEmployeeData;
        $per_page = 30;
        $aData['aPresent'] = (new Collection($aData['aPresent']))->paginate($per_page);
        $aData['aPakage'] = PakageModel::get_Pakages()->get();
        return view('reports.present_report', $aData);
    }

    public function present_absent_export_excel(){
        return Excel::download(new PresentAbsentExport(), 'PresentAbsent.xlsx');
    }


    public function hours_report(Request $request){
        $aData = array();
        $aData['search'] = '';
        if($request->form_type=='search_frm'){
            session(['Pakage_id' => $request->Pakage_id]);
            session(['start_date' => $request->start_date]);
            session(['end_date' => $request->end_date]);
        }

        $aData['Pakage_id'] = session::has('Pakage_id') ? session::get('Pakage_id') : 0;
        $aData['start_date'] = session::has('start_date') ? session::get('start_date') : date('d-m-Y',strtotime('-1 month'));
        $aData['end_date'] = session::has('end_date') ? session::get('end_date') :date('d-m-Y');

        if(!empty($aData['Pakage_id'])){ 
            $aData['search'] = 1;
        }

        $totalHours = ReportModel::getTotalHours($aData);
        foreach($totalHours as $val){
            $hours = ReportModel::secondsToTime($val->hours);
            $aData['totalHours'] = $hours['h'].':'.$hours['m'];
        }

        $aEmployeeData = ReportModel::getHoursData($aData);
        
        foreach($aEmployeeData as $key=>$data){
			$hours = ReportModel::secondsToTime($data->hours);
			$aEmployeeData[$key]->hours = $hours['h'].':'.$hours['m'];
            
            $sql = 'select Pakage_name from admin_pakages where id ='.$data->Pakage_id;
            $rRes = DB::select( DB::raw($sql));	
            $aEmployeeData[$key]->Pakage_name = $rRes[0]->Pakage_name; 
		}
		
        $aData['aHours'] = $aEmployeeData;
        $per_page = 30;
        $aData['aHours'] = (new Collection($aData['aHours']))->paginate($per_page);
        $aData['aPakage'] = PakageModel::get_Pakages()->get();
        return view('reports.hours_report', $aData); 
    }

    public function hours_wise_export_excel(){
        return Excel::download(new HoursWiseExport(), 'HoursWise.xlsx');
    }

    public function days_report(Request $request){

        $aData = array();
        $aData['search'] = '';
        if($request->form_type=='search_frm'){
            session(['Pakage_id' => $request->Pakage_id]);
            session(['start_date' => $request->start_date]);
            session(['end_date' => $request->end_date]);
        }

        $aData['Pakage_id'] = session::has('Pakage_id') ? session::get('Pakage_id') : 0;
        $aData['start_date'] = session::has('start_date') ? session::get('start_date') : date('d-m-Y',strtotime('-1 month'));
        $aData['end_date'] = session::has('end_date') ? session::get('end_date') :date('d-m-Y');

        if(!empty($aData['Pakage_id'])){ 
            $aData['search'] = 1;
        }

        $aEmployeeData = ReportModel::getDaysData($aData);
        $total_hours = 0;
		$total_employees = 0;
		foreach($aEmployeeData as $key=>$data){
			$total_hours += $data->total;
			$total_employees += $data->employee_count;
			$hours = ReportModel::secondsToTime($data->total);
			$aEmployeeData[$key]->hours = $hours['h'].':'.$hours['m'];
		}

		$hours = ReportModel::secondsToTime($total_hours);
		$aData['total_hours'] = $hours['h'].':'.$hours['m'];
		$aData['total_employees'] = $total_employees;
		$aData['aDays'] = $aEmployeeData;
        $per_page = 30;
        $aData['aDays'] = (new Collection($aData['aDays']))->paginate($per_page);
        $aData['aPakage'] = PakageModel::get_Pakages()->get();
        return view('reports.days_report', $aData);
    }

    public function days_wise_export_excel(){
        return Excel::download(new DaysWiseExport(), 'DaysWise.xlsx');
    }

    public function single_employee(Request $request){
        $aData = array();
        $aData['search'] = '';
        if($request->form_type=='search_frm'){
            session(['emp_id' => $request->emp_id]);
            session(['start_date' => $request->start_date]);
            session(['end_date' => $request->end_date]);
        }

        $aData['emp_id'] = session::has('emp_id') ? session::get('emp_id') : "";
        $aData['start_date'] = session::has('start_date') ? session::get('start_date') : date('d-m-Y',strtotime('-1 month'));
        $aData['end_date'] = session::has('end_date') ? session::get('end_date') :date('d-m-Y');

        if(!empty($aData['emp_id'])){ 
            $aData['search'] = 1;
        }
        
        $totalHours = ReportModel::getSingleHours($aData);
        
        foreach($totalHours as $val){
            $hours = ReportModel::secondsToTime($val->hours);
            $aData['totalHours'] = $hours['h'].':'.$hours['m'];
        }
        
        $aData['aEmployee'] = ReportModel::getEmployee($aData);
		$aEmployeeData = ReportModel::getSingleData($aData);

        foreach($aEmployeeData as $key=>$data){
		
			if($data->inward_datetime < $data->outword_datetime AND !empty($data->inward_datetime) AND !empty($data->outword_datetime)){	
				$hours = ReportModel::secondsToTime($data->hours);
				$aEmployeeData[$key]->hours = $hours['h'].':'.$hours['m'];
			}	
			else{
				$aEmployeeData[$key]->hours = '';
            }
		}
		$aData['aEmployeeData'] = $aEmployeeData;
        $per_page = 30;
        $aData['aEmployeeData'] = (new Collection($aData['aEmployeeData']))->paginate($per_page);        
        return view('reports.single_employee', $aData);
    }

    public function employee_wise_export_excel(){
        return Excel::download(new EmployeeWiseExport(), 'EmployeeWise.xlsx');
    }

    
    public function invalid_entries(Request $request){
        $aData = array();
        $aData['search'] = '';
        if($request->form_type=='search_frm'){
            session(['Pakage_id' => $request->Pakage_id]);
            session(['start_date' => $request->start_date]);
            session(['end_date' => $request->end_date]);
        }
        if($request->form_type=='validate'){
			ReportModel::reportValidate($request->problem);
            echo "<script>alert('Employee attendance marked as valid successfully')</script>";
        }
        $aData['Pakage_id'] = session::has('Pakage_id') ? session::get('Pakage_id') : "";
        $aData['start_date'] = session::has('start_date') ? session::get('start_date') : date('d-m-Y',strtotime('-1 month'));
        $aData['end_date'] = session::has('end_date') ? session::get('end_date') :date('d-m-Y');

        if(!empty($aData['Pakage_id'])){ 
            $aData['search'] = 1;
        }
        $aEmployeeData = ReportModel::getInvalidData($aData);

        foreach($aEmployeeData as $key=>$data){
			$aEmployeeData[$key]->current_day = date('Ymd',$data->attendance_date);
			if($data->inward_datetime < $data->outword_datetime AND !empty($data->inward_datetime) AND !empty($data->outword_datetime)){	
				$hours = ReportModel::secondsToTime($data->hours);
				$aEmployeeData[$key]->hours = $hours['h'].':'.$hours['m'];
                
			}	
			else{
				$aEmployeeData[$key]->hours = '';
            }
		}
		
        $aData['aInvalid'] = $aEmployeeData;
        $per_page = 30;
        $aData['aInvalid'] = (new Collection($aData['aInvalid']))->paginate($per_page);
        $aData['aPakage'] = PakageModel::get_Pakages()->get();
        return view('reports.invalid_entries', $aData);
    }

    public function invalid_entries_export_excel(){
        return Excel::download(new InvalidEntriesExport(),'InvalidEntries.xlsx');
    }
   
    public function datewise_report(Request $request){
        $aData = array();
        $aData['search'] = '';
        if($request->form_type=='search_frm'){
            session(['Pakage_id' => $request->Pakage_id]);
            session(['start_date' => $request->start_date]);
            session(['end_date' => $request->end_date]);
        }
        
        $aData['Pakage_id'] = session::has('Pakage_id') ? session::get('Pakage_id') : 0;
        $aData['start_date'] = session::has('start_date') ? session::get('start_date').' 00:00:00': date('d-m-Y', strtotime('-1 month'));
        $aData['end_date'] = session::has('end_date') ? session::get('end_date') .' 23:59:59' : date('d-m-Y 23:59:59');

        if(!empty($aData['Pakage_id'])){ 
            $aData['search'] = 1;
        }

        $aData['totalPresent'] = ReportModel::datewisePresentAbsentCount($aData);  

        
        $per_page = 30;
        // dd( $aData['totalPresent']);
        $aData['totalPresent'] = (new Collection( $aData['totalPresent']))->paginate($per_page);
       
        $aData['aPakage'] = PakageModel::get_Pakages()->get();
        return view('reports.datewise_report', $aData);  
    }

    public function search(Request $request){
        session(['Pakage_id' => $request->Pakage_id]);
        session(['start_date' => $request->start_date]);
        session(['end_date' => $request->end_date]);
        return redirect()->route('reports.list');  
    }

    public function date_wise_export_excel(){
        return Excel::download(new DateWiseExport(), 'DateWise.xlsx');
    }
}
