<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use App\Models\Module;
use App\Libraries\Mysecurity;
use Illuminate\Support\Facades\Session;
use Illuminate\Support\Facades\DB;



class ModuleController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function __construct()
    {
        $module_link = request()->segments(0);
        if(isset($module_link[0])){
            $module_link = $module_link[0];
            $this->mysecurity = new Mysecurity();
            $this->module_id = $this->mysecurity->get_module_id_by_name($module_link);
            $this->middleware(function ($request, $next){
                $this->mysecurity->check_access(Session::get('logged_in')['id'],$this->module_id,1);
        
                return $next($request);
            });
        }
    }
    public function index()
    {
        $aData = array();        
        $aData['aModules'] = Module::all()->where('active','=',1);
        return view('modules.index',$aData);
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        return view('modules.create');
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        $this->validate($request,[
            'module_name' => 'required',
            'module_caption' => 'required',
            'module_link' => 'required'
        ]);

        $module = new Module;

        $request->active = (isset($request->active)) ? $request->active : 0;
        $module->module_name = $request->module_name;
        $module->module_caption = $request->module_caption;
        $module->module_link = $request->module_link;
        $module->module_fa_icon = $request->module_fa_icon ? $request->module_fa_icon : '';   
        $module->active = 1;
        $module->last_updated_date = strtotime('now'); 
        $module->save();         

        return redirect()->route('modules.index')->with('success', 'Module Added Successfully'); 
    }

    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
        $aData = array(); 
        $aData['aModule'] = Module::find($id); 
        //return $aData;
        return view('modules.edit',$aData);
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, $id)
    {
       
       $this->validate($request,[
            'module_name' => 'required',
            'module_caption' => 'required',
            'module_link' => 'required'
        ]);

        $request->active = (isset($request->active)) ? $request->active : 0;

        $module = Module::findOrFail($id);
        $module->module_name = $request->module_name;
        $module->module_caption = $request->module_caption;
        $module->module_link = $request->module_link;
        $module->module_fa_icon = $request->module_fa_icon ? $request->module_fa_icon : '';
        $module->last_updated_date = strtotime('now'); 
        $module->save();          

        return redirect()->route('modules.index')->with('success', 'Module Updated Successfully'); 
        
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy($id)
    {
        Module::find($id)->delete();
        return back()->with('success', 'Module deleted successfully');
    }

    public function change_status(Request $request)
    {
        $module = Module::find($request->id);
        $module->active = $request->status;
        $module->save();
        return response()->json(['success'=>'User status change successfully.']);
    }

    public function sort_order(Request $request){

        $id = $request->id;
        $sort_order = $request->sort_order;
        Module::where('id', $id)->update(['sort_order' => $sort_order]);
        return response()->json(['success'=>'module sort order successfully.']);
    }
}
