<?php

namespace App\Http\Controllers;
use Illuminate\Http\Request;
use App\Models\RouteModel;
use App\Models\WardModel;
use App\Models\VehicleModel;
use App\Libraries\Mysecurity;
use Illuminate\Support\Facades\Session;
use Illuminate\Support\Facades\DB;
use App\Imports\RoutedetailsImport;
use App\Imports\RoutepolygonImport;
use App\Imports\PrabhagboundaryImport;
use Maatwebsite\Excel\Facades\Excel;
use App\Http\Controllers\Controller;

class RouteController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function __construct()
    {
        
        $this->route_model = new RouteModel();

        $module_link = request()->segments(0);
        if(isset($module_link[0])){
            $module_link = $module_link[0];
            $this->mysecurity = new Mysecurity();
            $this->module_id = $this->mysecurity->get_module_id_by_name($module_link);
            $this->middleware(function ($request, $next){
                $this->mysecurity->check_access(Session::get('logged_in')['id'],$this->module_id,1);
        
                return $next($request);
            });
        }
    }

    public function clear_search(){
        session::forget('Pakage_id');
        session::forget('ward_id');
        session::forget('route_id');
        return redirect()->route('route_master.index');
    }

    public function index(Request $request)
    {
        $this->mysecurity->check_access(Session::get('logged_in')['id'],$this->module_id,1);
        $aData = array();
        $aData['search'] = '';
        if($request->form_type=='search_form'){  
                // dd($request->Pakage_id);     
            session::put(['Pakage_id'  => $request->Pakage_id]);
            session::put(['ward_id'      => $request->ward_id]);
            session::put(['route_id'     => $request->route_id]);
        }

        $aData['Pakage_id'] = session::has('Pakage_id') ? session::get('Pakage_id') : 0;
        $aData['ward_id']     = session::has('ward_id') ? session::get('ward_id') : 0;
        $aData['route_id']    = session::has('route_id') ? session::get('route_id') : 0;
        if(!empty($aData['Pakage_id']) || !empty($aData['ward_id']) || !empty($aData['route_id'])){
             $aData['search'] = 1;
        }
       
        $aData['result'] = RouteModel::get($aData);
        $aData['route'] = VehicleModel::get_routes()->orderBy('route_name', 'ASC')->get();
        $aData['Pakages'] = VehicleModel::get_Pakages()->orderBy('Pakage_name')->get();
        $aData['wards'] = VehicleModel::get_wards()->orderBy('ward_no', 'ASC')->get();
        $aData['aVehicleNumber'] = VehicleModel::get_vehicle()->orderBy('vehicle_number', 'ASC')->get();

        $sql = "select vs.id,concat(vs.name,'(',vt.type,')') as name from vehicle_subtypes vs left join vehicle_types vt on vt.id = vs.vehicle_type_id  where 1=1 order by vs.name asc";
        $aData['aTypes'] = DB::select($sql);
        return view('route_master.list',$aData);
    }

    public function search(Request $request){
        session::put(['Pakage_id' => $request->Pakage_id]);
        session::put(['ward_id'     => $request->ward_id]);
        session::put(['route_id'     => $request->route_id]);
        return redirect()->route('route_master.index');  
    }

    public function create()
    {   
        $this->mysecurity->check_access(Session::get('logged_in')['id'],$this->module_id,2);
        $aData['Pakages'] = VehicleModel::get_Pakages()->orderBy('Pakage_name', 'ASC')->get();
        $aData['wards'] = VehicleModel::get_wards()->orderBy('ward_no', 'ASC')->get();
        return view('route_master.create',$aData);
    }

    public function store(Request $request)
    {   
        $this->mysecurity->check_access(Session::get('logged_in')['id'],$this->module_id,2);
        $this->validate($request,[
            'route_name'   => 'required|unique:route_master,route_name',
            'Pakage_id'  => 'required',
        ]);

        $module = new RouteModel;


        $module->route_name     = $request->route_name;
        $module->Pakage_id    = $request->Pakage_id ? $request->Pakage_id : 0;  
        $module->ward_id        = $request->ward_id ? $request->ward_id : 0;
        $module->ward_id_old    = $request->ward_id_old ? $request->ward_id_old : 0;
        $module->save();         

        return redirect()->route('route_master.index')->with('success', 'Record Added Successfully'); 
    }

    public function edit($id)
    {   
        $this->mysecurity->check_access(Session::get('logged_in')['id'],$this->module_id,2);
        $aData = array();
        $aData['result']    = RouteModel::find($id); 
        $aData['Pakages'] = VehicleModel::get_Pakages()->orderBy('Pakage_name', 'ASC')->get();
        $aData['wards']     = VehicleModel::get_wards()->orderBy('ward_no', 'ASC')->get();
       
        return view('route_master.edit',$aData);
    }

    public function update(Request $request, $id)
    {   
        $this->mysecurity->check_access(Session::get('logged_in')['id'],$this->module_id,2);
        $this->validate($request,[
            'route_name' => 'required|unique:route_master,route_name,'. $id .'id',
            'Pakage_id'  => 'required',
        ]);

        $module = RouteModel::findOrFail($id);
        $module->route_name     = $request->route_name;
        $module->Pakage_id    = $request->Pakage_id ? $request->Pakage_id : 0;  
        $module->ward_id        = $request->ward_id ? $request->ward_id : 0;
        $module->save();            

        return redirect()->route('route_master.index')->with('success', 'Record Updated Successfully');  
    }

    public function destroy($id)
    { 
      $this->mysecurity->check_access(Session::get('logged_in')['id'],$this->module_id,2);
      $users = RouteModel::findOrFail($id);
      $users -> delete();
      return redirect()->route('route_master.index')->with('error','Record Deleted Successfully');
    }

    public function get_wards(Request $request)
    {
        $aWards = WardModel::where('Pakage_id',$request->Pakage_id)->get();
        $assigWards = WardModel::where('id',$request->ward_id)
                                ->where('Pakage_id',$request->Pakage_id)
                                ->select('ward_no')->first();
        $userWards = array();
        if(isset($assigWards->ward_no)){
            $userWards = explode(',',$assigWards->ward_no);
        }
        
        $html = '';
        $html .= '<option value="0">--- Select Wards ---</option>';  
        foreach($aWards as $val){
            $selected = '';           
            if(in_array($val->id,$userWards)){
                $selected = 'selected';
            } 
          
            $html .= '<option '.$selected.' value="'.$val->id.'">'.$val->ward_name.'</option>';
        }
        return $html;
    }
    
    //--------- Add checkpoint ----------------------
    public function add_checkpoint(Request $request){
       if($request->form_type == 'set_as_checkpoint'){  

         $checkpoint_data['location'] = $request->location;
         $checkpoint_data['expected_time'] = $request->expected_time;
         $checkpoint_data['checkpoint_number'] = $request->checkpoint_no;
         $checkpoint_data['route_id'] = $request->route_id;
         $checkpoint_data['latitude'] = $request->latitude;
         $checkpoint_data['longitude'] = $request->longitude;
       
          DB::table('route_checkpoints')->insert($checkpoint_data);
          return redirect()->route('route_master.view_route', $request->route_id);
       }
    } 
    
    //--------- Edit checkpoint ----------------------
    public function edit_checkpoint(Request $request){
        if($request->form_type == 'update_checkpoint'){  
           DB::table('route_checkpoints')->where('id', $request->chkpt_id)->update(['location' => $request->location, 'expected_time' => $request->expected_time, 'checkpoint_number' => $request->checkpoint_no]); 
           return redirect()->route('route_master.view_route', $request->route_id);

        }
    } 
    
    //--------- Delete checkpoint ----------------------
    public function remove_checkpoint(Request $request, $id, $route_id){
        //dd($route_id);
       DB::table('route_checkpoints')->where('id', $id)->delete();
       return redirect()->route('route_master.view_route', $request->route_id);	
    }
    
    public function get_checkpoint(Request $request)
    {
       //return $request; die;
       $chkpt_id = $request->chkpt_id;
       $aCheckpoint = RouteModel::get_checkpoint_details($chkpt_id);
       $route_checkpoints = RouteModel::get_checkpoint($aCheckpoint->route_id);
       //dd($route_checkpoints);
       $aCheckpoints = config('custom.checkpoint_numbers');
       
        $aCheckpoints1 = array();
		foreach($aCheckpoints as $key => $value){
			if(!in_array($key,$route_checkpoints) || $key == $aCheckpoint->checkpoint_number){
				$aCheckpoints1[$key] = $value;
			}
		}
		$aData['aCheckpoints'] = $aCheckpoints1;

        $options = '';
        $options = '<option value="">Select Checkpoint</option>';
        foreach($aData['aCheckpoints'] as $key => $value){
            $selected = '';
            if($key == $aCheckpoint->checkpoint_number){
                $selected = 'selected';
            }
            $options .= '<option value="'.$key.'" '.$selected.'>'.$value.'</option>';
        }
    
        $html = '';
        $html ='<tr>
					    <td class="caption">Arrival Time</td>
						<td><input type="time" class="form-control" name="expected_time" id="expected_time" value='.$aCheckpoint->expected_time.' ></td> 
					</tr>
					<tr>
						<td class="caption">Location</td>
						<td><input type="text" class="form-control" name="location" id="location" value="'.$aCheckpoint->location.'" autocomplete="off"></td>
					</tr>
					<tr>
						<td class="caption">Checkpoint Number</td>
						<td>
							<select class="form-control" name="checkpoint_no" id="checkpoint_no">
							'.$options.'
							</select>
						</td>
					</tr> 
					<tr>	
						
						<input type="hidden" name="form_type" value="update_checkpoint"/>
						<td><input type="hidden" name="chkpt_id" id="chkpt_id" value="'.$aCheckpoint->id.'"/>
                        <input type="hidden" name="route_id" id="route_id" value="'.$aCheckpoint->route_id.'"/>
                        </td>
						<td><input type="submit" class="btn btn-primary btn-flat" name="submit" value="Update" /></td>
					</tr>';	
        return $html;
    }


    public function view_route(Request $request, $iRouteId,$route_name){
   
        $aData = array();
        
        //------- set mark as default show point
        $aData['vehicle_master'] = RouteModel::get_vehicle_master();
        $pockets = RouteModel::get_route_polygon($iRouteId);
        
        $aData['get_lat_lng'] = '';
        if(!empty($pockets)){
          $aData['get_lat_lng'] = $pockets[0]->points;
        }
         
         

        $aData['iRouteId'] = $iRouteId;
        $aData['route_name'] = base64_decode($route_name);
        
        return view('route_master.view_route', $aData);
    }
    // Added by monu 06-11-2023
    public function get_pocket_data(Request $request){
        
        $iRouteId    = $request->input('route_id');

        $aData['pockets'] = RouteModel::get_route_polygon($iRouteId);
        $aData['pocket_checkpoints'] = RouteModel::get_route_polygon_checkpoint($iRouteId);

        echo json_encode($aData);
    }

    public function pocket_store(Request $request){
        $txt_lat_long       = $request->input('txt_lat_long');
        $route_id           = $request->input('route_id');
        $number_of_pass_id  = $request->input('number_of_pass_id');
        $pocket_name        = str_replace(array("'", "\""),"",$request->input('pocket_name'));
        $total_distance     = $request->input('total_distance');

        $insert_data =  array(
                        'route_id'          => $route_id,
                        'points'            => $txt_lat_long,
                        'number_of_pass_id' => $number_of_pass_id,
                        'pocket_name'       => $pocket_name,
                        'total_distance'    => $total_distance
                    );

        $result = RouteModel::insert_data_pocket($insert_data);
    

        if($result){

            echo 1;
            
        }else{

            echo 0;
        }
    }

    public function marker_store(Request $request){

        $route_id          = $request->input('route_id');
        $route_polygon_id  = $request->input('route_polygon_id');
        $new_latitude      = $request->input('txt_lat');
        $new_longitude     = $request->input('txt_lng');

        $sSQL = 'SELECT count(id) as count FROM `route_polygon_checkpoint` WHERE (((acos(sin(('.$new_latitude.'*pi()/180)) * sin((`latitude`*pi()/180)) + cos(('.$new_latitude.'*pi()/180)) * cos((`latitude`*pi()/180)) * cos((('.$new_longitude.'- `longitude`)*pi()/180))))*180/pi())*60*1.1515*1609.344) < 200 AND route_polygon_id = '.$route_polygon_id.'';

        $rResult = DB::select($sSQL);

        if($rResult[0]->count < 1){

            $insert_data =  array(
                            'route_id'         => $route_id,
                            'route_polygon_id' => $route_polygon_id,
                            'latitude'         => $new_latitude,
                            'longitude'        => $new_longitude
                        );

            $result = RouteModel::insert_data_marker($insert_data);
        

            if($result){

                echo 2;
                
            }else{

                echo 0;
            }
        }else{

                echo 1;
            }
    }

    public function pocket_delete(Request $request){

        $id     = $request->input('id');

        $delete_data =  array(
                        'id' => $id
                    );

        $result = RouteModel::deleted_data_pocket($delete_data);

        if($result){

            echo 1;
            
        }else{

            echo 0;
        }
    }

    // Added by monu 12-01-2024
    public function clear_route_current_date(Request $request,$vehicle_id){
        
        $vehicle_id     = $vehicle_id;
        $current_day  = date('Ymd');
        //$current_day  = date('20231111');

        $query = 'DELETE FROM daily_route_details WHERE vehicle_number= "'.$vehicle_id.'" AND current_day= "'.$current_day.'"';

        $Result = DB::delete($query);

        if($Result){

            return redirect()->route('route_master.index')->with('success', 'Record Deleted Successfully');

        }else{

            return redirect()->route('route_master.index')->with('error', 'Record Not Found!');
        }

        
    }

    public function pocket_update(Request $request){

        $id              = $request->input('id');
        $pocket_name     = str_replace(array("'", "\""),"",$request->input('pocket_name'));;
        $number_of_pass  = $request->input('number_of_pass');
        $total_distance  = $request->input('total_distance');

        //echo '<pre>';print_r($edit_pocket_id);die();

        $update_data =  array(
                        'id' => $id,
                        'pocket_name' => $pocket_name,
                        'number_of_pass' => $number_of_pass,
                        'total_distance' => $total_distance
                    );

         $result = RouteModel::update_data_pocket($update_data);

        if($result){

            echo 1;
            
        }else{

            echo 0;
        }
    }
    public function pocket_edit_update(Request $request){

        $id              = $request->input('id');
        $edit_pocket_id  = $request->input('edit_pocket_id');

        //echo '<pre>';print_r($edit_pocket_id);die();

        $update_data =  array(
                        'id'     => $id,
                        'points' => $edit_pocket_id
                    );

         $result = RouteModel::edit_update_data_pocket($update_data);

        if($result){

            echo 1;
            
        }else{

            echo 0;
        }
    }

    public function marker_delete(Request $request){

        $id     = $request->input('id');

        $delete_data =  array(
                        'id' => $id
                    );

        $result = RouteModel::deleted_data_marker($delete_data);

        if($result){

            echo 1;
            
        }else{

            echo 0;
        }
    }
    public function all_checkpoint_delete(Request $request){

        $id     = $request->input('id');

        $delete_data =  array(
                        'route_polygon_id' => $id
                    );

        $result = RouteModel::deleted_data_all_checkpoint($delete_data);

        if($result){

            echo 1;
            
        }else{

            echo 0;
        }
    }

    public function get_travelled_route(Request $request){

        $sel_vehicle_id  = $request->input('sel_vehicle');
        $txt_date        = $request->input('txt_date');

        if(!empty($sel_vehicle_id) && !empty($txt_date)){

            $folder_name = date('Ymd',strtotime($txt_date));

            $filePath = public_path('vehicle_details') . '/'.$folder_name.'/'.$sel_vehicle_id.'.json';

            if(file_exists($filePath)){

                   $travelled_json = '['.file_get_contents($filePath).']';

                    $a_return['travelled_route'] = array();
                    $a_return['map_center_lat'] = 0;
                    $a_return['map_center_lng'] = 0;

                    if(!empty($travelled_json)){
                        $a_travelled_route = json_decode($travelled_json);
                        //echo '<pre>';print_r($a_travelled_route);
                        foreach($a_travelled_route as $o_point){
                            if(empty($a_return['map_center_lat']) && empty($a_return['map_center_lng'])){
                                $a_return['map_center_lat'] = $o_point->latitude;
                                $a_return['map_center_lng'] = $o_point->longitude;                        
                            }

                            $a_return['travelled_route'][] = [
                                'lat' => $o_point->latitude,
                                'lng' => $o_point->longitude
                            ];
                        }
                    }

                    $json_response = '[]';
                    if(!empty($a_return)){
                        $json_response = json_encode($a_return);
                    }

                    return $json_response;

                   //echo '<pre>';print_r($json_response);
                //echo '1';
            }else{
                echo '2';
            }
        }
    }

    // public function import_route(Request $request){

    //     Excel::import(new RoutedetailsImport($request->route_id), request()->file('route_file'),\Maatwebsite\Excel\Excel::XLSX);
    //     return redirect()->route('route_master.view_route', $request->route_id)->with('success', 'Route updated Successfully');	
    // }

    // public function import_pocket(Request $request){

    //      Excel::import(new RoutepolygonImport($request->route_id), request()->file('pocket_file'),\Maatwebsite\Excel\Excel::XLSX);
    //     return redirect()->route('route_master.view_route', $request->route_id)->with('success', 'Pocket updated Successfully');	
    // }


    
    public function import_boundary(Request $request){

        Excel::import(new PrabhagboundaryImport($request->route_id), request()->file('boundary_file'),\Maatwebsite\Excel\Excel::XLSX);
       return redirect()->route('route_master.view_route', $request->route_id)->with('success', 'Boundary updated Successfully');	
   }

    public function view_all_routes(Request $request){
      //dd('ssss');
      $aData = array();

      $aData['ward_id'] = 0;
	  $aData['vehicle_id'] = 0;
	  $aData['search'] = 0;
	  $aData['show_routes'] = 'checked';
	  $aData['show_pockets'] = '';

      if($request->form_type=='show_hide_frm'){ 
        //dd($request);
        $show_routes  = $request->show_routes;   
        $show_pockets = $request->show_pockets;

        if(!empty($show_routes)){
            $aData['show_routes'] = 'checked';			
        }else{
            $aData['show_routes'] = '';
        }
        
        if(!empty($show_pockets)){
            $aData['show_pockets'] = 'checked';			
        }else{
            $aData['show_pockets'] = '';
        }
      }
      
      if($request->form_type=='search_form'){ 
         session::put(['Pakage_id'  => $request->Pakage_id]);
         session::put(['ward_id'      => $request->ward_id]);
         session::put(['date'         => $request->date]);
      }

      $aData['Pakage_id'] = session::has('Pakage_id') ? session::get('Pakage_id') : 6;
      $aData['ward_id']     = session::has('ward_id') ? session::get('ward_id') : 0;
      if(!empty($aData['Pakage_id']) || !empty($aData['ward_id'])){
         $aData['search'] = 1;
      }

      $aSearch = array(
        'Pakage_id' => $aData['Pakage_id'],
        'ward_id'     => $aData['ward_id']
      );

      $aData['aRoutes'] = RouteModel::get_all_routes($aSearch);
      foreach($aData['aRoutes'] as $key => $aVal){
         $aVal['route_pocket'] = RouteModel::get_route_polygon($aVal['route_id']);
         $aData['aRoutes'][$key] = $aVal;
      }
     
       $aData['Pakages']    = VehicleModel::get_Pakages()->orderBy('Pakage_name', 'ASC')->get();
       $aData['wards']        = VehicleModel::get_wards()->orderBy('ward_no', 'ASC')->get();
    
       return view('route_master.all_routes', $aData);
    }

    public function all_clear_search(Request $request){
        session::forget('Pakage_id');
        session::forget('ward_id');
        return redirect()->route('route_master.view_all_routes');  
    }

    public function update_vehicle_route_calender(Request $request)
    {

        //echo '<pre>';print_r($request->all());

        $id                 = $request->input('edit_id');
        $vehicle_number     = $request->input('vehicle_number');
        $type_id            = $request->input('type_id');
        $start_date         = date('Ymd',strtotime($request->input('start_date')));
        $repeat_days_num    = $request->input('repeat_days_num');
        $vehicle_assign_day = strtotime('now');
        //echo '<pre>';print_r($edit_pocket_id);die();

        $update_data =  array(
                        'vehicle_id'          => $vehicle_number,
                        'sub_type_id'         => $type_id,
                        'start_date'          => $start_date,
                        'vehicle_assign_day'  => $vehicle_assign_day,
                        'repeat_days_num'     => $repeat_days_num,
                        'id'     => $id
                    );

        //echo '<pre>';print_r($update_data);die;

         $result = RouteModel::edit_update_vehicle_route($update_data);

        if($result){

            return redirect()->route('route_master.index')->with('success', 'Record Updated Successfully');
            
        }else{

            return redirect()->route('route_master.index')->with('error', 'Record Not Updated Successfully');
        }
    }
    public function remove_vehicle_route_calender(Request $request)
    {
        $id       = $request->input('route_id');

        $data_arr =  array(
                        'vehicle_id'  => 0,
                        'sub_type_id' => 0,
                        'start_date'  => '',
                        'repeat_days_num' => 0,
                        'id'     => $id
                    );

        // echo '<pre>';print_r($data_arr);die;

         $result = RouteModel::remove_vehicle_route_calender($data_arr);

        if($result){

            echo 1;
            
        }else{

            echo 0;
        }
    }
}
