<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use App\Models\RouteDeviationModel;
use App\Models\VehicleModel;
use App\Models\RouteModel;
use App\Libraries\Mysecurity;
use Illuminate\Support\Facades\Session;
use Illuminate\Support\Facades\DB;

class RouteDeviationNewController extends Controller
{
    public function __construct()
    {   
        $module_link = request()->segments(0);
        if(isset($module_link[0])){
            $module_link = $module_link[0];
            $this->mysecurity = new Mysecurity();
            $this->module_id = $this->mysecurity->get_module_id_by_name($module_link);
            $this->middleware(function ($request, $next){
                $this->mysecurity->check_access(Session::get('logged_in')['id'],$this->module_id,1);
        
                return $next($request);
            });
        }
    }

    public function clear_search(){
        session::forget('date');
        session::forget('vehicle_id');
        return redirect()->route('route_deviation.index',[0,0]);
    }


    public function index(Request $request, $vehicle_id = 0, $date = 0)
    {   
        //dd($vehicle_id);
        $this->mysecurity->check_access(Session::get('logged_in')['id'],$this->module_id,1);
        $date = (!empty($date)) ? $date : date('d-m-Y',strtotime('now'));
        if(!empty($vehicle_id)){
            // if(is_int($vehicle_id)){
            //     session::put(['vehicle_id'  => $vehicle_id]);
            //     session::put(['date'        => $date]);
            // }            
            return redirect()->route('route_deviation.view_route',[$vehicle_id,$date]);
		}
    
         $aData['vehicle_id'] = $vehicle_id;
         $aData['date']       = $date;
		$route_flag = 0;
        $aData['vehicle']   = VehicleModel::get_vehicle()->orderBy('vehicle_number', 'ASC')->get();
        return view('route_deviation.routes', $aData);
    }

    public function load_route(Request $request)
    {
        //$aData['aRoutes'] = array();
        $post = array(
            'vehicle_id'    =>  $request->vehicle_id,
            'date'          =>  $request->date,
        );

        $aData['aRoutes'] = RouteDeviationModel::get_routes($post);

        foreach($aData['aRoutes'] as $aRoute){
            if(!empty($aRoute['actual_route'])){
                $aData['route_flag'] = 1;
            }
        }

        //echo '<pre>';print_r($aData);die();

        echo json_encode($aData);
        //return response()->json(array('msg'=> $aData), 200);
    }

    public function view_route(Request $request, $vehicle_id = 0, $date = 0)
    {
        $this->mysecurity->check_access(Session::get('logged_in')['id'],$this->module_id,1);
        $aData = array();
        $aData['vehicle_id'] = $vehicle_id;
        $aData['date']       = (!empty($date)) ? $date : date('d-m-Y',strtotime('now'));
        $aData['vehicle']   = VehicleModel::get_vehicle()->orderBy('vehicle_number', 'ASC')->get();
        return view('route_deviation.routes', $aData);
    }

    public function load_checkpoint_arrival(Request $request)
    {
        $post_arr = array(
            'date'         =>  date('Ymd',strtotime($request->date)),
            'polygon_id'   =>  $request->polygon_id,
            'i_no'         =>  $request->i_no
        );

        //echo '<pre>';print_r($post_arr);die();

        $r_result   = RouteDeviationModel::get_checkpoint_arrival($post_arr);
       
        $aData = [];
        $html = '';
        if(!empty($r_result)){
               foreach ($r_result as $value) {
                   
                 $pass = $value->pass;
                  $html .= '<b>Pass</b> : '.$pass.' <b>Date</b> : '.date('d-m-Y',$value->actual_reach_time).' <b>Time</b> : '.date('H:i:s',$value->actual_reach_time).'<br>';
               }
               
               $aData['html'] = $html;
               $aData['route_flag'] = 1;
        }else{
               $aData['route_flag'] = 0;
        }
        
        //echo '<pre>';print_r($aData);die;
        echo json_encode($aData);
        
    }
    public function default_all_route()
    {   
        $aData = array();

        $sSQL = 'SELECT rp.*,rm.route_name FROM route_polygon rp LEFT JOIN route_master rm ON rp.route_id = rm.id  WHERE 1=1';

        $rRes = DB::select( DB::raw($sSQL));

        $sSQL2 = 'SELECT rpc.* FROM route_polygon_checkpoint rpc WHERE 1=1';

        $rRes2 = DB::select( DB::raw($sSQL2));
        
        $aData['pockets'] = $rRes;
        
        $aData['pocket_checkpoints'] = $rRes2;

        //echo '<pre>';print_r($aData['pocket_checkpoints']);die();

        return view('route_deviation.default_route_all',$aData);
    }

    public function default_all_route_new()
    {   
        $aData = array();

        return view('route_deviation.default_route_all_new',$aData);
    }

    public function get_all_route()
    {  
        $aData = array();

        $sSQL = 'SELECT rm.id,rm.route_name FROM route_master rm WHERE 1=1';

        if(Session::has('logged_in.superuser') && Session::get('logged_in.superuser') !=1){
            $session_div = Session::get('Pakages'); 
            if(!empty($session_div)){
                $str_div = implode(',',$session_div);
                $sSQL .= ' AND rm.Pakage_id IN ('.$str_div.')';
            }
        }

        $rRes = DB::select( DB::raw($sSQL));

        $aData['route_all'] = $rRes;

        echo json_encode($aData);

        //return view('route_deviation.default_route_all_new',$aData);
    }

    public function get_pocket_data_all(Request $request)
    { 

        $aData = array();

        $route_id = $request->input('route_id');


        $sSQL = 'SELECT rp.*,rm.route_name FROM route_polygon rp LEFT JOIN route_master rm ON rp.route_id = rm.id  WHERE 1=1';


        if($route_id !=0){
           
           $sSQL .= ' AND rp.route_id='.$route_id;
        }

        // echo $sSQL;die();

        $rRes = DB::select( DB::raw($sSQL));
        $aData['route_id'] = $route_id;
        $aData['route_all'] = $rRes;

        echo json_encode($aData);

        
    }
  
}
