<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use App\Models\PakageModel;
use App\Models\SupervisorModel;
use App\Models\VehicleModel;
use Illuminate\Support\Facades\Session;
use App\Libraries\Mysecurity;
use Illuminate\Support\Facades\DB;

class SupervisorController extends Controller 
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function __construct()
    {
        $module_link = request()->segments(0);
        if (isset($module_link[0])) {
            $module_link = $module_link[0];
            $this->mysecurity = new Mysecurity();
            $this->module_id = $this->mysecurity->get_module_id_by_name($module_link);
            $this->middleware(function ($request, $next) {
                $this->mysecurity->check_access(Session::get('logged_in')['id'], $this->module_id, 1);

                return $next($request);
            });
        }
    }
    
    public function clear_search(){
       
        session::forget('Pakage_id');
        session::forget('supervisors_name');
        return redirect()->route('supervisors.index');
     }

    public function index(Request $request)
    {   
        $this->mysecurity->check_access(Session::get('logged_in')['id'],$this->module_id,1);
        $aData = array();        
        $aData['search'] = '';

        if($request->form_type=='search_frm'){            
            session::put(['Pakage_id'       => $request->Pakage_id]);
            session::put(['supervisors_name'  => $request->supervisors_name]);
        }

        $aData['Pakage_id']    = session::has('Pakage_id') ? session::get('Pakage_id') : '';
        $aData['supervisors_name'] = session::has('supervisors_name') ? session::get('supervisors_name') : '';
        if(!empty($aData['Pakage_id']) || !empty($aData['supervisors_name'])){
           $aData['search'] = 1;
        }
        
        $aData['aSupervisor'] = SupervisorModel::get($aData)->orderBy('id', 'DESC')->paginate(config('custom.page_limit')); 
        $aData['Pakages'] = VehicleModel::get_Pakages()->orderBy('Pakage_name', 'ASC')->get();                       
        return view('supervisors.list', $aData);
    }

    public function search(Request $request){
        session(['Pakage_id'       => $request->Pakage_id]);
        session(['supervisors_name'  => $request->supervisors_name]);
        return redirect()->route('supervisors.index'); 
    }

    public function create()
    {
        $this->mysecurity->check_access(Session::get('logged_in')['id'],$this->module_id,2);
        $aData = array();
        $aData['aPakage'] = VehicleModel::get_Pakages()->orderBy('Pakage_name', 'ASC')->get();
        //$aData['aVehicleNumber'] = VehicleModel::get_vehicle()->orderBy('vehicle_number', 'ASC')->get(); 
        return view('supervisors.create', $aData);
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        $this->mysecurity->check_access(Session::get('logged_in')['id'],$this->module_id,2);
        $this->validate($request,[
            'Pakage_id' => 'required',
            'supervisor_name' => 'required',
            'user_name' => 'required',
            'password' => 'required|confirmed|min:5',
            'gender' => 'required'
        ]);

        if(!empty($request->user_name)){
            $result = SupervisorModel::where('Pakage_id', $request->Pakage_id)->get('user_name');
            $data = array();
            foreach ($result as $val){
                $data[] = $val->user_name;
            }

            // $result1 = SupervisorModel::where('vehicle_id', $request->vehicle_number)->get('vehicle_id');
            // $data1 = array();
            // foreach ($result1 as $val){
            //     $data1[] = $val->vehicle_id;
            // }

            if(in_array($request->user_name,$data)){
                    return redirect()->back()->with('error', 'User Name Already Exist In this Pakage ');
            }else{
                $supervisor = new SupervisorModel();
                $supervisor->Pakage_id = $request->Pakage_id;
                $supervisor->supervisor_name = $request->supervisor_name;
                $supervisor->gps_device_number    = $supervisor->gps_device_number ? $supervisor->gps_device_number : 0;
                $supervisor->user_name = $request->user_name;
                $supervisor->password = md5($request->password);
                $supervisor->gender = $request->gender;
                $supervisor->created = strtotime('now');
                $supervisor->updated_at = 0;
                $supervisor->save();
                return redirect()->route('supervisors.index')->with('success', 'Supervisor Added Successfully');
            }
        }
    }

    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
        $this->mysecurity->check_access(Session::get('logged_in')['id'],$this->module_id,2);
        $aData = array();
        $aData['aPakage'] = PakageModel::all(); 
        $aData['aVehicleNumber'] = VehicleModel::get_vehicle()->orderBy('vehicle_number', 'ASC')->get(); 
        $aData['aSupervisor'] =SupervisorModel::find($id);
       // $aData['avehicle'] =SupervisorModel::find($id);
        return view('supervisors.edit', $aData);
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, $id)
    {
        $this->mysecurity->check_access(Session::get('logged_in')['id'],$this->module_id,2);
        $this->validate($request,[
            'Pakage_id' => 'required',
            'supervisor_name' => 'required',
            'user_name' => 'required',
            'gender' => 'required'
        ]);

            // if($request->vehicle_number != 0){
            //     $vehicle_number = $request->vehicle_number;
            //     $sSQL = 'select count(id) as rec_count from supervisor_masters where vehicle_id = '.$vehicle_number.' and id != '.$id.' ';
            //     $rRes = DB::select( DB::raw($sSQL), array());
            //     $edit_vehicle_count = $rRes[0]->rec_count;
            // }else{
            //     $edit_vehicle_count = 0;
            // }
            
            if(!empty($request->user_name)){
                $user_name = $request->user_name;
                $sSQL1 = 'select count(id) as rec_user_count from supervisor_masters where user_name = "'.$user_name.'" and id != '.$id.' ';
                //echo $sSQL1; die;
                $rRes1 = DB::select( DB::raw($sSQL1), array());
                $edit_user_count = $rRes1[0]->rec_user_count;
            }else{
                $edit_user_count = 0; 
            }
                
                if($edit_user_count == 1){
                    return redirect()->back()->with('error', 'User Name Already Exist');
                }else{    
                    $supervisor = SupervisorModel::findOrFail($id);
                    $supervisor->Pakage_id = $request->Pakage_id;
                    $supervisor->supervisor_name = $request->supervisor_name;
                    $supervisor->gps_device_number    = $request->gps_device_number ? $request->gps_device_number : 0;  
                    $supervisor->user_name = $request->user_name;
                    if($request->password){
                        $supervisor->password = md5($request->password);
                    }
                    $supervisor->gender = $request->gender;
                    $supervisor->updated_at = strtotime('now');
                    $supervisor->save();
                    return redirect()->route('supervisors.index')->with('success', 'Supervisor updated Successfully');
                }
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy($id)
    {
        $this->mysecurity->check_access(Session::get('logged_in')['id'],$this->module_id,2);
        SupervisorModel::find($id)->delete();
        DB::table('supervisor_device_details')->where('supervisor_id',$id)->delete();
        DB::table('supervisor_assign_ward')
        ->where('supervisor_id', $id)
        ->update(['is_deleted'=> 1 ]);
        return back()->with('success', 'Supervisor Deleted Successfully');
    }

    public function supervisor_devices($id){
        $this->mysecurity->check_access(Session::get('logged_in')['id'],$this->module_id,2);
        $aData = array();
        $aData['aAllDevices'] = DB::table('supervisor_device_details')->where('supervisor_id', $id)->get();
        $aData['aSupervisor'] =SupervisorModel::find($id);
        return view('supervisors.supervisor_devices', $aData);
    }

    public function supervisor_submit_devices(Request $request){
        $this->mysecurity->check_access(Session::get('logged_in')['id'],$this->module_id,2);
        $this->validate($request,[
            'user_name' => 'required',
            'device_id' => 'required|unique:supervisor_device_details,device_id,',
           
        ]);
        
        DB::table('supervisor_device_details')->insert(['user_name'=>$request->user_name,
                                                        'supervisor_id'=>$request->supervisor_id,
                                                        'device_id'=>$request->device_id,
                                                        'created'=>strtotime('now')
                                                    ]);
        
        return back()->with('success', 'Supervisor devices added successfully');
    }

    public function remove_devices(Request $request){
        DB::table('supervisor_device_details')->where('id',$request->device_id)->where('supervisor_id',$request->supervisor_id)->delete();
        return response()->json(['success'=>'Supervisor devices deleted successfully.']);
    }

    public function supervisor_devices_edit($id, $sup_id){
        $aData['aDevices'] =  DB::table('supervisor_device_details')->where('id', $id)->where('supervisor_id', $sup_id)->get();
        return view('supervisors.supervisor_device_edit', $aData);
    }

    public function supervisor_devices_update(Request $request,$id){
        $this->validate($request,[
            'user_name' => 'required',
            'device_id' => 'required|unique:supervisor_device_details,device_id,'. $id .'id',
        ]);
        $aData['aDevices'] =  DB::table('supervisor_device_details')
                                ->where('id', $id)
                                ->where('supervisor_id', $request->supervisor_id)
                                ->update(['user_name'=>$request->user_name,
                                        'device_id'=>$request->device_id,
                                ]);
        return redirect('/supervisors/devices/'. $request->supervisor_id)->with('success', 'Supervisor devices updated successfully');

    }

    public function supervisors_change_active_status(Request $request){
        // dd("here");
        $this->mysecurity->check_access(Session::get('logged_in')['id'],$this->module_id,2);
        $Supervisor = SupervisorModel::find($request->id);
        $Supervisor->active= $request->status;
        $Supervisor->save();
        return response()->json(['success'=>'Supervisor status change successfully.']);
    }

    public function supervisor_assign_ward($id,$edit_id = 0){
        //dd($id);
        $aData = array();
        $aData['aResult'] = SupervisorModel::get_assign_supervisor($id);
        //print_r($aData['aResult']); die;
        if(!empty($edit_id)){
            $aData['aSupedit'] =  DB::table('supervisor_assign_ward')->where('id', $edit_id)->where('supervisor_id', $id)->get();

            //print_r($aData['aSupedit']); die;
        }
        $aData['aSupervisor'] = SupervisorModel::find($id);
        $aData['wards'] = VehicleModel::get_wards()->orderBy('ward_no', 'ASC')->get();
        return view('supervisors.supervisor_assign_ward',$aData);
    }

    public function supervisor_add_wards(Request $request){
       // dd($request);
        $this->mysecurity->check_access(Session::get('logged_in')['id'],$this->module_id,2);
        $this->validate($request,[
           'ward_id' => 'required',
        ]);
        

        if(!empty($request->ward_id) && !empty($request->supervisor_id)){
            $ward_id = $request->ward_id;
            $supervisor_id = $request->supervisor_id;
            $sSQL1 = 'select count(id) as rec_count from supervisor_assign_ward where ward_id = "'.$ward_id.'" and supervisor_id = '.$supervisor_id.' and is_deleted = 0';
            //echo $sSQL1; die;
            $rRes1 = DB::select( DB::raw($sSQL1), array());
            $add_count = $rRes1[0]->rec_count;
        }else{
            $add_count = 0; 
        }
         
        if($add_count == 1){
            return redirect()->back()->with('error', 'Ward Name Already Exist');
        }else{ 
            DB::table('supervisor_assign_ward')->insert(['supervisor_id'=>$request->supervisor_id,
                                                       'Pakage_id'=>$request->Pakage_id,
                                                       'ward_id'=>$request->ward_id
                                                    ]);
            return back()->with('success', 'Record Added Successfully');
        }
    }

    public function supervisor_edit_wards(Request $request,$sup_id,$id){
        // dd($request);
        $this->mysecurity->check_access(Session::get('logged_in')['id'],$this->module_id,2);

        if(!empty($request->ward_id) && !empty($request->supervisor_id)){
            $ward_id = $request->ward_id;
            $supervisor_id = $request->supervisor_id;
            $sSQL1 = 'select count(id) as rec_count from supervisor_assign_ward where ward_id = "'.$ward_id.'" and supervisor_id = '.$supervisor_id.' and id != '.$id.' and is_deleted = 0';
            //echo $sSQL1; die;
            $rRes1 = DB::select( DB::raw($sSQL1), array());
            $edit_count = $rRes1[0]->rec_count;
        }else{
            $edit_count = 0; 
        }
        
        if($edit_count == 1){
            return redirect()->back()->with('error', 'Ward Name Already Exist');
        }else{ 
            $aData['aWads'] =  DB::table('supervisor_assign_ward')
                                ->where('id', $id)
                                ->where('supervisor_id', $sup_id)
                                ->update(['ward_id'=>$request->ward_id
                                ]);

            return redirect('/supervisors/assignward/'.$sup_id.'/0')->with('success', 'Record Updated Successfully');
        }
    }

    public function remove_wards(Request $request){
       //dd($request);
        $aData['aWads'] =  DB::table('supervisor_assign_ward')
        ->where('id', $request->ward_id)
        ->where('supervisor_id', $request->supervisor_id)
        ->update(['is_deleted'=> 1 ]);

        return response()->json(['success'=>'Record Deleted Successfully.']);
    }
}
    