<?php

namespace App\Http\Controllers;
use Illuminate\Http\Request;
use App\Models\VehicleModel;
use App\Libraries\Mysecurity;
use Illuminate\Support\Facades\Session;
use Illuminate\Support\Facades\DB;


class VehicleController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function __construct()
    {
        $module_link = request()->segments(0);
        if(isset($module_link[0])){
            $module_link = $module_link[0];
            $this->mysecurity = new Mysecurity();
            $this->module_id = $this->mysecurity->get_module_id_by_name($module_link);
            $this->middleware(function ($request, $next){
                $this->mysecurity->check_access(Session::get('logged_in')['id'],$this->module_id,1);
        
                return $next($request);
            });
        }
    }

    public function clear_search(){
        session::forget('Pakage_id');
        session::forget('vehicle_number');
       // session::forget('ward_id');
        return redirect()->route('vehicle_master.index');
    }

    public function index(Request $request)
    {
        $this->mysecurity->check_access(Session::get('logged_in')['id'],$this->module_id,1);
        $aData = array();
        $aData['search'] = '';
        if($request->form_type=='search_form'){     
            session::put(['Pakage_id'     => $request->Pakage_id]);
            session::put(['vehicle_number'  => $request->vehicle_number]);
        }

        $aData['Pakage_id'] = session::has('Pakage_id') ? session::get('Pakage_id') : 0;
        $aData['vehicle_number']     = session::has('vehicle_number') ? session::get('vehicle_number') : 0;
        if(!empty($aData['Pakage_id']) || !empty($aData['vehicle_number'])){
             $aData['search'] = 1;
        }
       
        $aData['result']    = VehicleModel::get($aData)->orderBy('id', 'ASC')->paginate(config('custom.page_limit')); 
        $aData['Pakages'] = VehicleModel::get_Pakages()->orderBy('Pakage_name', 'ASC')->get();
        $aData['vehicle']   = VehicleModel::get_vehicle()->orderBy('vehicle_number', 'ASC')->get();

        return view('vehicle_master.list',$aData);
    }

    public function search(Request $request){
        session::put(['Pakage_id'     => $request->Pakage_id]);
        session::put(['vehicle_number'  => $request->vehicle_number]);
        return redirect()->route('vehicle_master.index'); 
    }

    public function create(){
        $this->mysecurity->check_access(Session::get('logged_in')['id'],$this->module_id,2);
        $aData['Pakages'] = VehicleModel::get_Pakages()->orderBy('Pakage_name', 'ASC')->get();
        return view('vehicle_master.create',$aData);
    }
    /**
     * Written by Gaurav to change vehicle status to disable
     */
    public function change_status(Request $request){
        $this->mysecurity->check_access(Session::get('logged_in')['id'],$this->module_id,2);
        if($request->id){
            $sql = 'UPDATE vehicle_master SET is_active='.$request->status.' WHERE id='.$request->id;
            if(Session::has('logged_in.superuser') && Session::get('logged_in.superuser') !=1){
                $session_div = Session::get('Pakages'); 
                
                if(!empty($session_div)){
                    $session_div = implode(',',$session_div);
                    
                    //$query->whereIn('Pakage_id',$session_div);    
                    $sql .= ' AND Pakage_id IN ('.$session_div.')';
                }else {
                    $sql .= '1=2';    
                }
            }       
            DB::select( DB::raw($sql ), array());
            echo 1;
        }else{
            echo 0;
        }
    }
    

    public function store(Request $request){
        $this->mysecurity->check_access(Session::get('logged_in')['id'],$this->module_id,2);

        $this->validate($request,[
            'vehicle_number' => 'required|unique:vehicle_master,vehicle_number',
            'device_number'  => 'required|unique:vehicle_master,device_number',
            'sim_number'     => 'required|unique:vehicle_master,sim_number',
            'rfid_number'    => 'unique:vehicle_master,rfid_number',
            //'tank_size'      => 'required',
        ]);

        $module = new VehicleModel;
        $module->vehicle_number = $request->vehicle_number;
        $module->device_number  = $request->device_number;
        $module->sim_number     = $request->sim_number;
        $module->rfid_number    = $request->rfid_number ? $request->rfid_number : 0;  
        $module->Pakage_id    = $request->Pakage_id ? $request->Pakage_id : 0;  
        $module->description    = $request->description ? $request->description : ''; 
        // $module->tank_size      = $request->tank_size; 
        // $module->max_voltage    = $request->max_voltage ? $request->max_voltage : 0;
        $module->save();         

        return redirect()->route('vehicle_master.index')->with('success', 'Record Added Successfully'); 
    }

    public function edit($id){
        $this->mysecurity->check_access(Session::get('logged_in')['id'],$this->module_id,2);

        $aData = array();
        $aData['result'] = VehicleModel::find($id); 
        $aData['Pakages'] = VehicleModel::get_Pakages()->orderBy('Pakage_name', 'ASC')->get();
        return view('vehicle_master.edit',$aData);
    }

    public function update(Request $request, $id){
        $this->mysecurity->check_access(Session::get('logged_in')['id'],$this->module_id,2);

        $this->validate($request,[
            
            'vehicle_number' => 'required|unique:vehicle_master,vehicle_number,'. $id .'id',
            'device_number'  => 'required|unique:vehicle_master,device_number,'. $id .'id',
            'sim_number'     => 'required|unique:vehicle_master,sim_number,'. $id .'id',
            'rfid_number'    => 'required|unique:vehicle_master,rfid_number,'. $id .'id',

            //'tank_size'      => 'required',
        ]);

        $module = VehicleModel::findOrFail($id);
        $module->vehicle_number = $request->vehicle_number;
        $module->device_number  = $request->device_number;
        $module->sim_number     = $request->sim_number;
        $module->rfid_number    = $request->rfid_number ? $request->rfid_number : 0; 
        $module->Pakage_id    = $request->Pakage_id ? $request->Pakage_id : 0; 
        $module->description    = $request->description ? $request->description : '';
        // $module->tank_size      = $request->tank_size; 
        // $module->max_voltage    = $request->max_voltage ? $request->max_voltage : 0;
        $module->save();          
        return redirect()->route('vehicle_master.index')->with('success', 'Record Updated Successfully'); 
    }

    public function destroy($id){
      $this->mysecurity->check_access(Session::get('logged_in')['id'],$this->module_id,2);  
      $users = VehicleModel::findOrFail($id);
      $users -> delete();
      return redirect()->route('vehicle_master.index')->with('error','Record Deleted Successfully');
   }
}
